function tracearc, file, biassec, datasec
dx=10        ; width of window for gaussian fitting
dy=20        ; height of window for peak finding and gaussian fitting
dmin=15      ; minimum separation between peaks
thresh=300   ; threshold for peak finding
lam0=2459.36   ; first guess for wavelength zeropoint
dlam=1.6288    ; first guess for dispersion
dlam2=7.176e-5 ; first guess for second order term
order=5        ; order of 1D wavelength solution
order2=[5,3]  ; order of 2D wavelength solution
bottomedge=5   ; bottom row at which fit arclines
topedge=2279   ; top row at which fit arclines

; by eye good Ids to get first guess, do not delete
;xfit=[839.82997, 1177.8789, 1312.0852, 1426.0507, 1681.7903, 1872.1178, 2722.1170, 3150.6621, 3189.0364]
;yfit=[3888.6460, 4471.4771, 4713.1431, 4921.9292, 5400.5620, 5764.4180, 7438.899, 8300.3262, 8377.6074]


; ------------------- read line list
readcol, '~/idl/typhoonpipe/linelist.dat', lines

; ------------------- read arc lamp file
data0=mrdfits(file, 0, hdr, /DSCALE)
data1=oscansub(data0, biassec, datasec)
nx=(size(data1))[1]
ny=(size(data1))[2]
data=dblarr(nx, ny)
for i=0, ny-1 do begin & auxcont=medsmooth(data1[*,i], 600) & data[*,i]=data1[*,i]-auxcont & endfor ; ---- remove nasty continuum
mwrfits, data, 'arc.fits', /create

; ------------------ collapse dy pixels at bittom of the chip for peak
;                    finding 
auxdata=dindgen(nx)
for i=0, nx-1 do auxdata[i]=median(data[i,bottomedge:bottomedge+fix(dy/2.)])
dauxdata=deriv(auxdata)
xarr=dindgen(nx)


; ------------------- find peaks above threshold and not saturated
peak0=fltarr(nx)
auxpeak0=fltarr(nx)+1
for i=0, nx-2 do if (dauxdata[i] ge 0 and dauxdata[i+1] lt 0 and auxdata[i] ge thresh) then peak0[i]=1 
for i=0, nx-1 do if (total(peak0[max([0,i-dmin]):min([nx-1,i+dmin])]) gt 1 or auxdata[i] ge 5e4) then auxpeak0[i]=0
peak0=peak0*auxpeak0
peak1=where(peak0 eq 1, npeak)
peak=dblarr(npeak)

; ------------------ gaussian fit to find peak centroid 
for i=0, npeak-1 do begin 
   auxx=xarr[max([0,floor(peak1[i]-dx/2.)]):min([nx-1,ceil(peak1[i]+dx/2.)])] 
   auxy=auxdata[max([0,floor(peak1[i]-dx/2.)]):min([nx-1,ceil(peak1[i]+dx/2.)])] 
   fit=gaussfit(auxx, auxy, coef, NTERMS=4, ESTIMATES=[auxdata[peak1[i]], peak1[i], 2, 0]) 
   peak[i]=coef[1] 
;   plot, auxx, fit 
;   oplot, auxx, auxy, psym=2, color=250 
;   pause 
endfor

; --------------- plot for user aproval
window, 0, xsize=1200, ysize=700
!p.multi=[0,1,2,0,0]
plot, xarr, auxdata, /ylog, yrange=[1e2, 1e5], xrange=[0,floor(nx/2.)], xstyle=1
oplot, peak, peak/peak*1e4, psym=1, color=150
oplot, [0,1e4], [thresh, thresh], color=250
plot, xarr, auxdata, /ylog, yrange=[1e2, 1e5], xrange=[floor(nx/2.), nx], xstyle=1
oplot, peak, peak/peak*1e4, psym=1, color=150
oplot, [0,1e4], [thresh, thresh], color=250
pause
!p.multi=[0,1,1,0,0]

; --------------- first guess at peak wavelength and association
lamguess=lam0+dlam*peak+dlam2*peak^2
peaklam=dblarr(npeak)
for i=0, npeak-1 do peaklam[i]=lines[where(abs(lines-lamguess[i]) eq min(abs(lines-lamguess[i])))]

; --------------- fit wavelength solution and ask user to remove bad lines

peakorig=peak
peaklamorig=peaklam
sw=1
while (sw eq 1) do begin
   sol=poly_fit(peak, peaklam, order, yfit=peaklamfit)
   window, 0, xsize=1200, ysize=300
   plot, peak, peaklam-peaklamfit, psym=2
   oplot, [0,1e4], [0,0], linestyle=2
   for i=0, n_elements(peak)-1 do xyouts, peak[i]+20, (peaklam[i]-peaklamfit[i]), strtrim(string(i+1),2), color=250
   print, 'WAVELENGTH SOLUTION RMS= ', stddev(peaklam-peaklamfit)
   print, 'ENTER ORDER'
   read, order
   print, 'ENTER ARRAY OF PEAK #s TO REMOVE SEPARATED BY COMAS (eg 3,6,8) OR 0 TO ACCEPT SOLUTION, OR -1 TO START OVER'
   remove=''
   read, remove
   remove=fix(strsplit(remove, ',', /EXTRACT)) 
   if (remove[0] eq 0) then sw=0
   if (remove[0] eq -1) then begin 
      peak=peakorig
      peaklam=peaklamorig
   endif
   if (remove[0] ne 0 and remove[0] ne -1) then begin
      remove=remove-1
      accept=where(peak)
      for i=0, n_elements(remove)-1 do accept=accept[where(accept ne remove[i])]
      peak=peak[accept]
      peaklam=peaklam[accept]
   endif
endwhile



; ------------- TRACE ARCLINES ACROSS Y DIRECTION ----------------
npeak=n_elements(peak)
xpeak=dblarr(ny, npeak)-666
xpeak[bottomedge,*]=peak

window, 0, xsize=600, ysize=600

for j=bottomedge+1, topedge-1 do begin
   print, "DOING ROW ", j+1, " of ", ny
   auxdata=dindgen(nx)
   for k=0, nx-1 do auxdata[k]=median(data[k,max([0,floor(j-dy/2.)]):min([ny-1,ceil(j+dy/2)])])
   for i=0, npeak-1 do begin
      auxx=xarr[max([0,floor(xpeak[j-1,i]-dx/2.)]):min([nx-1,ceil(xpeak[j-1,i]+dx/2.)])] 
      auxy=auxdata[max([0,floor(xpeak[j-1,i]-dx/2.)]):min([nx-1,ceil(xpeak[j-1,i]+dx/2.)])] 
      fit=gaussfit(auxx, auxy, coef, NTERMS=4, ESTIMATES=[auxdata[round(xpeak[j-1,i])], xpeak[j-1,i], 2, 0]) 
      xpeak[j,i]=coef[1] 

;         plot, auxx, fit, xrange=[xpeak[0,i]-50, xpeak[0,i]+50], yrange=[1e1, 1e5], xstyle=1, ystyle=1, /ylog
;         oplot, auxx, auxy, psym=2, color=150
  ;       pause
   endfor
endfor


; -------------- fit bivariate polynomial lambda(x, y) -------------

xfit=dblarr(n_elements(where(xpeak ne -666)))-666
yfit=dblarr(n_elements(where(xpeak ne -666)))-666
lamfit=dblarr(n_elements(where(xpeak ne -666)))-666

cntr=0l
for i=0, npeak-1 do begin &    for j=0, ny-1, 10 do begin &      if (xpeak[j,i] ne -666) then begin &         xfit[cntr]=xpeak[j,i] &             yfit[cntr]=j &          lamfit[cntr]=peaklam[i]    &         cntr=cntr+1 &              endif  &     endfor    & endfor

sel=where(xfit ne -666)
xfit=xfit[sel]
yfit=yfit[sel]
lamfit=lamfit[sel]


save, xfit, filename='xfitarc.idl'
save, yfit, filename='yfitarc.idl'
save, lamfit, filename='lamfitarc.idl'


;fit1=poly_xy_fit(xfit, yfit, lamfit, order2, ufit1, usig1)
;fit2=poly_xy_fit(lamfit, yfit, xfit, order2, ufit2, usig2)


fit1=sfit(transpose([[xfit], [yfit], [lamfit]]), 6, kx=coef1, /IRREGULAR, /MAX_DEGREE)
;fit2=sfit(transpose([[xfit], [lamfit], [yfit]]), 5, kx=coef2, /IRREGULAR, /MAX_DEGREE)


;junk=sfiteval(xfit, yfit, 6, kx=coef1, /IRREGULAR, /MAX_DEGREE)


print, "RMS= ", stddev(lamfit-fit1)

!p.multi=[0,1,2,0,0]
window, 0, xsize=800, ysize=800
plot, xfit, lamfit-fit1, psym=3, xtitle='X', ytitle=textoidl('\Delta\lambda')
plot, yfit, lamfit-fit1, psym=3, xtitle='Y', ytitle=textoidl('\Delta\lambda')
!p.multi=[0,1,1,0,0]

;!p.multi=[0,1,2,0,0]
;window, 0, xsize=800, ysize=800
;plot, lamfit, xfit-fit2, psym=3
;plot, yfit, xfit-fit2, psym=3
;!p.multi=[0,1,1,0,0]

print, 'HIT RETURN TO CONTINUE...'
pause


openw, lun, 'arctrace.xy', /get_lun
for i=0, npeak-1 do begin
   for j=0, ny-1 do begin
   if (xpeak[j,i] ne -666) then printf, lun, xpeak[j,i]+1, j+1
   endfor
endfor
free_lun, lun


;openw, lun, 'arcpoly.xy', /get_lun
;yarr=findgen(50)/(50.)*ny
;for i=0, npeak-1 do begin &   for j=0, 50-1 do begin &  auxx=poly_xy(peaklam[i], yarr[j], fit2) ;&  & printf, lun, auxx+1, yarr[j]+1 & endfor & endfor
;free_lun, lun


return, coef1

end
