; returns coefficients of bivariate polynomial yslit(x,y)

function tracehtm, file, biassec, datasec, xslit

dx=20. ; width of x window for peak search and gaussian fit
dy=10  ; height of y window for gaussian fit
nstep=20 ; step for tracing each peak
ledge=700 ; left edge down to which trace 
degree=[4,4] ; degree of distorsion polynomial fit
ystart=20 ; lowest row at which to find peak

data0=mrdfits(file, 0, hdr, /DSCALE)
data=oscansub(data0, biassec, datasec)

nx=(size(data))[1]
ny=(size(data))[2]

mxslit=round(median(xslit[*,0])) ; median x coordinate of slit
thresh=2.*median(data[mxslit,*]) ; threshhold for peak acceptance

; ------------- find peaks at slit position ---------------

auxdata=total(data[floor(mxslit-dx/2.):ceil(mxslit+dx/2.),*], 1)/dx
dauxdata=deriv(auxdata)
yarr=dindgen(ny)

peak0=fltarr(ny)
for i=0, ny-2 do if (dauxdata[i] ge 0 and dauxdata[i+1] lt 0 and auxdata[i] ge thresh and i gt ystart) then peak0[i]=1 
peak1=where(peak0 eq 1, npeak)
peak=fltarr(npeak)

for i=0, npeak-1 do begin
   auxx=yarr[floor(peak1[i]-dy/2.):ceil(peak1[i]+dy/2.)]
   auxy=auxdata[floor(peak1[i]-dy/2.):ceil(peak1[i]+dy/2.)]
   fit=gaussfit(auxx, auxy, coef, NTERMS=4, ESTIMATES=[auxdata[peak1[i]], peak1[i], 2, thresh/2.])
   peak[i]=coef[1]

;   plot, auxx, fit
;   oplot, auxx, auxy, psym=2, color=250
;   pause

endfor


loadct, 39
window, 0, xsize=1200, ysize=300
plot, auxdata, yrange=[thresh*1e-1,thresh*1e2], /ylog        
oplot, [0,1e4], [thresh, thresh], color=250   
oplot, peak, peak/peak*30.*thresh, psym=1, color=150                

print, "HIT RETURN TO CONTINUE..."
pause

; ------------- trace peaks across dispersion direction  ---------------


ypeak=dblarr(nx, npeak)
nopeak=dblarr(nx,npeak)
ypeak[mxslit,*]=peak


for i=0, npeak-1 do begin
   print, "TRACING PEAK ", i+1, " of ", npeak
   ; trace to the right
   for j=mxslit+1, nx-1 do begin
      auxdata0=total(data[floor(j-dx/2.):min([nx-1,ceil(j+dx/2.)]),*], 1)/dx
      auxthresh=2.*median(auxdata0) ; threshhold for peak acceptance
      auxx=yarr[max([0,floor(ypeak[j-1,i]-dy/2.)]):min([ny-1,ceil(ypeak[j-1,i]+dy/2.)])]
      auxy=auxdata0[max([0,floor(ypeak[j-1,i]-dy/2.)]):min([ny-1,ceil(ypeak[j-1,i]+dy/2.)])]
      if (max(auxy) ge auxthresh) then begin
         fit=gaussfit(auxx, auxy, coef, NTERMS=4, ESTIMATES=[auxdata0[ypeak[j-1,i]], ypeak[j-1,i], 2, auxthresh/2.])
         ypeak[j,i]=coef[1]
         
;         plot, auxx, fit, xrange=[ypeak[mxslit,i]-50, ypeak[mxslit,i]+50], yrange=[1e3, 1e5], xstyle=1, ystyle=1, /ylog
;         oplot, auxx, auxy, psym=2, color=150
;         oplot, [0,1e4], [auxthresh, auxthresh], color=250   
;         pause
      endif else begin
         ypeak[j,i]=ypeak[j-1,i]
         nopeak[j,i]=1
      endelse
   endfor 
 ; trace to the left
   for j=mxslit-1, ledge, -1 do begin
      auxdata0=total(data[floor(j-dx/2.):min([nx-1,ceil(j+dx/2.)]),*], 1)/dx
      auxthresh=2.*median(auxdata0) ; threshhold for peak acceptance
      auxx=yarr[floor(ypeak[j+1,i]-dy/2.):min([ny-1,ceil(ypeak[j+1,i]+dy/2.)])]
      auxy=auxdata0[floor(ypeak[j+1,i]-dy/2.):min([ny-1,ceil(ypeak[j+1,i]+dy/2.)])]
      if (max(auxy) ge auxthresh) then begin
         fit=gaussfit(auxx, auxy, coef, NTERMS=4, ESTIMATES=[auxdata0[ypeak[j+1,i]], ypeak[j+1,i], 2, auxthresh/2.])
         ypeak[j,i]=coef[1]
         
     ;    plot, auxx, fit, xrange=[ypeak[mxslit,i]-50, ypeak[mxslit,i]+50], yrange=[1e3, 1e5], xstyle=1, ystyle=1, /ylog
     ;    oplot, auxx, auxy, psym=2, color=150
     ;    oplot, [0,1e4], [auxthresh, auxthresh], color=250   
    ;     pause
      endif else begin
         ypeak[j,i]=ypeak[j+1,i]
         nopeak[j,i]=1
      endelse
   endfor
endfor
ypeak[where(nopeak eq 1)]=-666
ypeak[0:ledge,*]=-666


; -------------- fit bivariate polynomial yslit(x, y) -------------



xfit=dblarr(n_elements(where(ypeak ne -666)))-666
yfit=dblarr(n_elements(where(ypeak ne -666)))-666
ysfit=dblarr(n_elements(where(ypeak ne -666)))-666

cntr=0l
xslit=xslit[*,0]
yslit=dindgen(ny)


for i=0, npeak-1 do begin & for j=0, nx-1, 10 do begin &      if (ypeak[j,i] ne -666) then begin &          xfit[cntr]=j &          yfit[cntr]=ypeak[j,i] &   ysfit[cntr]=ypeak[xslit[round(ypeak[mxslit,i])],i]    &    cntr=cntr+1 &          endif  &      endfor    &    endfor

sel=where(xfit ne -666)
xfit=xfit[sel]
yfit=yfit[sel]
ysfit=ysfit[sel]


save, xfit, filename='xfit.idl'
save, yfit, filename='yfit.idl'
save, ysfit, filename='ysfit.idl'


;fit1=poly_xy_fit(xfit, yfit, ysfit, degree, ufit1, usig1)
;fit2=poly_xy_fit(xfit, ysfit, yfit, degree, ufit2, usig2)

ufit1=sfit(transpose([[xfit], [yfit], [ysfit]]), 4, kx=coef1, /IRREGULAR, /MAX_DEGREE)
ufit2=sfit(transpose([[xfit], [ysfit], [yfit]]), 4, kx=coef2, /IRREGULAR, /MAX_DEGREE)



print, "RMS= ", stddev(ysfit-ufit1)

!p.multi=[0,1,2,0,0]
window, 0, xsize=800, ysize=800
plot, xfit, ysfit-ufit1, psym=1
plot, yfit, ysfit-ufit1, psym=1
!p.multi=[0,1,1,0,0]



print, 'HIT RETURN TO CONTINUE...'
pause

mwrfits, data, 'htmmask.fits', /create
openw, lun, 'htmtrace.xy', /get_lun
for i=0, npeak-1 do begin
   for j=0, nx-1 do begin
      if (ypeak[j,i] ne -666) then printf, lun, j+1, ypeak[j,i]+1
   endfor
endfor
free_lun, lun

;openw, lun, 'htmpoly.xy', /get_lun
;xarr=findgen(50)/(50.)*nx
;for i=0, npeak-1 do begin 
;   auxy=sfiteval(xarr, ypeak[xslit[round(ypeak[mxslit,i])],i], 6, kx=coef2, /IRREGULAR, /MAX_DEGR;EE)  
;for j=0, 50-1 do printf, lun, xarr[j]+1, auxy[j]+1
;endfor

;for i=0, npeak-1 do begin &   for j=0, 50-1 do begin &  auxy=poly_xy(xarr[j], ypeak[xslit[round(ypeak[mxslit,i])],i], fit2) &  & printf, lun, xarr[j]+1, auxy+1 & endfor & endfor
;free_lun, lun




return, coef1

end
